"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const database_1 = require("./helpers/database");
const user_repository_1 = require("./helpers/modules/users/user.repository");
describe('ObjectionJS Repository Testing', function () {
    let userRepo;
    before(function () {
        return __awaiter(this, void 0, void 0, function* () {
            userRepo = new user_repository_1.UserRepository(yield database_1.Database.getInstance());
        });
    });
    beforeEach(function () {
        return __awaiter(this, void 0, void 0, function* () {
            yield database_1.Database.seed();
        });
    });
    describe('Read functions testing', function () {
        it('Should get one user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const user = yield userRepo.getOne({ age: 25 });
                assert_1.default.equal(25, user === null || user === void 0 ? void 0 : user.age);
                assert_1.default.equal('Ahmed', user === null || user === void 0 ? void 0 : user.name);
            });
        });
        it('Should get name only.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const user = yield userRepo.getOne({ age: 25 }, {
                    select: ['name'],
                });
                assert_1.default.deepEqual(user, {
                    name: 'Ahmed',
                });
            });
        });
        it('Should get one user with where null.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const user = yield userRepo.getOne({}, {
                    whereNull: ['email'],
                });
                assert_1.default.equal(30, user === null || user === void 0 ? void 0 : user.age);
                assert_1.default.equal('Ali', user === null || user === void 0 ? void 0 : user.name);
            });
        });
        it('Should get All user with where not null.', function () {
            var _a;
            return __awaiter(this, void 0, void 0, function* () {
                const users = yield userRepo.getAll({}, {
                    whereNotNull: ['email'],
                });
                assert_1.default.equal(2, users === null || users === void 0 ? void 0 : users.length);
                assert_1.default.equal('Adel', (_a = users[1]) === null || _a === void 0 ? void 0 : _a.name);
            });
        });
        it('Should not get one user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const user = yield userRepo.getOne({ age: 41 });
                assert_1.default.equal(undefined, user);
            });
        });
        it('Should get all users.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const users = yield userRepo.getAll({});
                assert_1.default.equal(3, users.length);
            });
        });
        it('Should get all users with condition.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const users = yield userRepo.getAll({ name: 'Ali' });
                assert_1.default.equal(1, users.length);
                assert_1.default.equal('Ali', users[0].name);
            });
        });
        it('Should get all users with where in condition.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const users = yield userRepo.getAll({}, { whereIn: [{ field: 'name', values: ['Ahmed', 'Ali'] }] });
                assert_1.default.equal(2, users.length);
            });
        });
    });
    describe('Write functions testing', function () {
        it('Should create user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const userToBeInserted = {
                    name: 'Akin',
                    age: 28,
                };
                yield userRepo.create(userToBeInserted);
                const user = yield userRepo.getOne(userToBeInserted);
                assert_1.default.equal(userToBeInserted.age, user === null || user === void 0 ? void 0 : user.age);
                assert_1.default.equal(userToBeInserted.name, user === null || user === void 0 ? void 0 : user.name);
            });
        });
        it('Should update user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const userBeforeUpdate = {
                    name: 'Ahmed',
                    age: 25,
                };
                const userToBeUpdated = {
                    name: 'Yamaç',
                    age: 29,
                };
                yield userRepo.update(userBeforeUpdate, userToBeUpdated);
                const updatedUser = yield userRepo.getOne(userToBeUpdated);
                const oldUser = yield userRepo.getOne(userBeforeUpdate);
                assert_1.default.equal(oldUser, undefined);
                assert_1.default.equal(userToBeUpdated.name, updatedUser === null || updatedUser === void 0 ? void 0 : updatedUser.name);
            });
        });
        it('Should delete user.', function () {
            return __awaiter(this, void 0, void 0, function* () {
                const userToBeDeleted = {
                    name: 'Ahmed',
                    age: 25,
                };
                yield userRepo.delete(userToBeDeleted);
                const oldUser = yield userRepo.getOne(userToBeDeleted);
                assert_1.default.equal(oldUser, undefined);
            });
        });
    });
});
